import io

import qrcode
from PIL import Image, ImageDraw, ImageFont, ImageOps
from qrcode.image.styledpil import StyledPilImage
from qrcode.image.styles.colormasks import RadialGradiantColorMask
from qrcode.image.styles.moduledrawers import RoundedModuleDrawer

from config import FONT_CONF, QR_CONF


def generate(rooms, color, client):
    def save_image(image, filename):
        output = io.BytesIO()
        image.save(output, "png")
        output.seek(0)
        client.upload(output, filename, "image/png")
        output.close()

    def single_room(room):
        print(f'🎨 Generating {".".join(room["path"])}.{room["name"]} ({room["id"]})')
        qr = qrcode.QRCode(error_correction=qrcode.constants.ERROR_CORRECT_H)

        qr.add_data(f'{QR_CONF["url"]}/public/{room["id"]}')

        image = qr.make_image(
            image_factory=StyledPilImage,
            module_drawer=RoundedModuleDrawer(),
            color_mask=RadialGradiantColorMask(
                back_color=(255, 255, 255), center_color=color, edge_color=(0, 0, 0)
            ),
            embeded_image_path=QR_CONF['logo'],
        )

        header = QR_CONF['header']

        def split(a, n):
            k, m = divmod(len(a), n)
            return (
                a[i * k + min(i, m) : (i + 1) * k + min(i + 1, m)] for i in range(n)
            )

        footer = "\n".join(
            [" - ".join(x) for x in list(split(room["path"] + [room["name"]], 3))]
        )

        expanded_image = ImageOps.expand(image, border=20, fill="white")

        # Add define a new font to write in the border
        big_font = ImageFont.truetype(FONT_CONF['path'], int(FONT_CONF['header']))
        small_font = ImageFont.truetype(FONT_CONF['path'], int(FONT_CONF['footer']))

        # Instantiate draw object & add desired text
        draw_object = ImageDraw.Draw(expanded_image)
        draw_object.text(xy=(60, 10), text=header, fill=(0, 0, 0), font=big_font)

        draw_object.text(
            xy=(60, expanded_image.height - 55),
            text=footer,
            fill=(0, 0, 0),
            font=small_font,
        )

        bordered = ImageOps.expand(expanded_image, border=10, fill=tuple(color))

        # Preview the image
        # bordered.show()

        # Save the image
        # bordered.save(f'out/{room["id"]}.png')
        save_image(bordered, f'{room["id"]}.png')
        return bordered

    images = list(map(single_room, rooms))

    w, h = images[0].size

    separation = 2

    # create big empty image with place for images
    new_image = Image.new(
        "RGB", (w * 2 + separation, h * 2 + separation), color="white"
    )

    # put images on new_image
    new_image.paste(images[0], (0, 0))
    new_image.paste(images[1], (w + separation, 0))
    new_image.paste(images[2], (0, h + separation))
    new_image.paste(images[3], (w + separation, h + separation))

    s_ids = multi(rooms)
    # save it
    print(f"🎨 Generating {s_ids}")
    # new_image.save(f'out/{"-".join(s_ids)}.png')
    save_image(new_image, f"{s_ids}.png")


def multi(rooms):
    ids = [int(room["id"]) for room in rooms]
    ids.sort()
    return "-".join(list(map(str, ids)))



import datetime
import io

import boto3
from botocore.client import Config

from config import S3_CONF


class S3_client:
    def __init__(self):
        self.client = boto3.client(
            "s3",
            aws_access_key_id=S3_CONF['access_key'],
            aws_secret_access_key=S3_CONF['secret_key'],
            endpoint_url=S3_CONF['endpoint'],
        )

        self.bucket = S3_CONF['bucket']

        response = self.client.list_buckets()

        if self.bucket in [b["Name"] for b in response["Buckets"]]:
            print(f"🪣 Bucket {self.bucket} already exists")
        else:
            print(f"🪣 Creating bucket {self.bucket}")
            self.client.create_bucket(Bucket=self.bucket)
            self.client.put_bucket_cors(
                Bucket=self.bucket,
                CORSConfiguration={
                    "CORSRules": [
                        {
                            "AllowedMethods": ["GET", "HEAD"],
                            "AllowedOrigins": [
                                "*",
                            ],
                            "ExposeHeaders": ["*"],
                            "AllowedHeaders": ["Content-Type", "Authorization"],
                        }
                    ]
                },
            )

    def upload(self, content, filename, mime_type):
        print(f"⬆️ Uploading file {filename}")
        self.client.upload_fileobj(
            content,
            self.bucket,
            filename,
            ExtraArgs={
                "ContentType": mime_type,
                "ACL": "public-read",
            },
        )

    def get(self, filename):
        print(f"⬇️ Downloading {filename}")
        output = io.BytesIO()
        self.client.download_fileobj(self.bucket, filename, output)
        return output.getvalue()

    def exists(self, filename):
        response = self.client.list_objects(Bucket=self.bucket)
        return "Contents" in response and filename in [
            item["Key"]
            for item in self.client.list_objects_v2(
                Bucket=self.bucket, Prefix=filename
            )["Contents"]
        ]



import json

from jinja2 import Environment, FileSystemLoader, PackageLoader, select_autoescape

from config import S3_CONF


def render(data):
    env = Environment(
        loader=FileSystemLoader("templates"), autoescape=select_autoescape()
    )
    template = env.get_template("tree.html")

    return template.render(
        data=data,
        s3={
            "endpoint": S3_CONF['endpoint'],
            "bucket": S3_CONF['bucket'],
            "tenant": S3_CONF['tenant'],
        },
    )


# data = json.load(open('out.json'))
# print(render(data))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['schedulesy_qrcode']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'boto3>=1.24.70,<2.0.0',
 'qrcode[pil]>=7.3.1,<8.0.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'schedulesy-qrcode',
    'version': '1.0.0',
    'description': '',
    'long_description': "# Générateur de QRCodes pour Schedulesy\n\n## Installation des dépendances\n\n### `Poetry`\n\nL'outil utilise `poetry` comme gestionnaire de dépendances. Voir la [procédure d'installation](https://python-poetry.org/docs/master/#installation) (privilégier une version de `python` >= 3.9 pour l'installation de `poetry`).\n\n``` \npoetry install\n```\n\n### `Pip`\n\nUne alternative est d'utiliser `pip` pour les dépendances. Des fichiers sont fournis pour les différents environnements.\n\nPour installer et exécuter :\n\n```\npip install -r requirements/common.txt\n```\n\nPour installer aussi les dépendances de développement : \n\n```\npip install -r requirements/dev.txt\n```\n\n## Usage\n\nVous devez créer un fichier de configuration en vous inspirant du fichier `config.ini.sample`.\n\n``` \npython main.py config.ini\n🔗 Connecting to ADE\n📖 Setting project\n💾 Fetching data\n🪣 Bucket example already exists\n⬇️ Downloading flat.json\n⬆️ Uploading file tree.json\n⬆️ Uploading file flat.json\n⬆️ Uploading file index.html\n```",
    'author': 'DIP - Université de Strasbourg',
    'author_email': 'dnum-dip@unistra.fr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
